SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

USE pubs
go

SELECT p.pub_name, t.title, a.au_lname
	FROM Publishers AS p
		INNER JOIN Titles AS t ON p.pub_id = t.pub_id
		INNER JOIN titleauthor AS ta ON ta.title_id = t.title_id
		INNER JOIN authors AS a ON a.au_id = ta.au_id
WHERE t.type= 'business'

-- Where is all the cost? (Highest number in showplan output...)
	-- Should be in titles table roughly 65% if no changes 
	-- have been made to the pubs sample database.

-- I recommend this strategy for all queries - break it down into each part with 
-- primary focus on the join conditions, search conditions and aggregates. If there's 
-- a group by please prioritize for the aggregate first. See the slides on indexing for 
-- aggregates.

-- SO - what does the titles table need in this query?
-- Search List		title 
-- Join Cond1		pub_id
-- Join Cond2		title_id
-- Search Arg		type

-- Create "test" Indexes with priority (first column - or high order element) 
-- given to the search argument or the join and come up with some test cases... 
-- Try out the query again and see which one it chooses! Always put the cols of the
-- select list last!!!! They are the least significant!

-- Prioritizing for sarg (then join cond1, then jc2, then select list)
	CREATE INDEX test1 ON titles(type, pub_id, title_id, title)
-- Prioritizing for sarg (then jc2, then jc1, then select list)
	CREATE INDEX test2 ON titles(type, title_id, pub_id, title)

-- Prioritizing for join(s) (jc1, sarg, jc2, select list)
	CREATE INDEX test3 ON titles(pub_id, type, title_id, title)
-- Prioritizing for join(s) (jc2, sarg, jc1, select list)
	CREATE INDEX test4 ON titles(title_id, type, pub_id, title)

-- Run the query again to see which one it chooses?

SELECT p.pub_name, t.title, a.au_lname
	FROM Publishers AS p
		INNER JOIN Titles AS t ON p.pub_id = t.pub_id
		INNER JOIN titleauthor AS ta ON ta.title_id = t.title_id
		INNER JOIN authors AS a ON a.au_id = ta.au_id
WHERE t.type= 'business'

-- Seems to like Test2 so drop the other indexes!

DROP INDEX titles.test1
--DROP INDEX titles.test2
DROP INDEX titles.test3
DROP INDEX titles.test4

-- Need to continue and see if you can tweak it more... 
-- Run the query again to where the highest cost is now?

SELECT p.pub_name, t.title, a.au_lname
	FROM Publishers AS p
		INNER JOIN Titles AS t ON p.pub_id = t.pub_id
		INNER JOIN titleauthor AS ta ON ta.title_id = t.title_id
		INNER JOIN authors AS a ON a.au_id = ta.au_id
WHERE t.type= 'business'
